@@ -1,18 +1,42 @@
+# 2011-05-15 CJS limited etaU to 20 to prevent overflow in binomial computations
+# 2011-03-09 CJS added prior to muTT (mean.muTT and sd.muTT) with defaults same a previously
+# 2011-02-17 CJS limited initial Pguess to between .01 and .99 to avoid taking logit of 0 or 1
+# 2011-02-17 CJS fixed initial values for theta; added as.matrix to deal with case of Delta.max=1
+# 2011-01-24 SB  added call to run.windows.openbugs and run.windows.winbugs
 # 2010-11-25 CJS output on progress of burnin and post-burnin phases
 # 2010-04-25 CJS fixed problems of init.logitP=+infinity if n1=m2=k which crapped out the lm() call
 # 2010-03-11 Added fixed values of logitP[j] to be provided by user.
 # 2010-03-02 SJB Created file.
 
-TimeStratPetersenNonDiagErrorNP <- function(title, prefix, time, n1, m2, u2,
+TimeStratPetersenNonDiagErrorNP <- function(title,
+                                            prefix,
+                                            time,
+                                            n1,
+                                            m2,
+                                            u2,
                                             jump.after=NULL,
-                                            logitP.cov=rep(1,length(u2)), logitP.fixed=rep(NA,length(u2)),
-                                            n.chains=3, n.iter=200000, n.burnin=100000, n.sims=2000, 
-                                            tauU.alpha=1, tauU.beta=.05, taueU.alpha=1, taueU.beta=.05,
-                                            Delta.max,tauTT.alpha=.1, tauTT.beta=.1,
-                                            mu_xiP=-2, tau_xiP=.6666, 
-                                            tauP.alpha=.001, tauP.beta=.001, 
-                                            debug=FALSE, debug2=FALSE, openbugs=TRUE, InitialSeed,
-                                            OPENBUGS.directory, WINBUGS.directory ){
+                                            logitP.cov=rep(1,length(u2)),
+                                            logitP.fixed=rep(NA,length(u2)),
+                                            n.chains=3,
+                                            n.iter=200000,
+                                            n.burnin=100000,
+                                            n.sims=2000,
+                                            tauU.alpha=1,
+                                            tauU.beta=.05,
+                                            taueU.alpha=1,
+                                            taueU.beta=.05,
+                                            Delta.max,
+                                            mean.muTT=rep(0,Delta.max),
+                                            sd.muTT=rep(sqrt(.666),Delta.max),
+                                            tauTT.alpha=.1,
+                                            tauTT.beta=.1,
+                                            mu_xiP=-2,
+                                            tau_xiP=.6666,
+                                            tauP.alpha=.001,
+                                            tauP.beta=.001,
+                                            debug=FALSE,
+                                            debug2=FALSE,
+                                            InitialSeed){
 #
 #  Fit the smoothed time-Stratified Petersen estimator with NON-Diagonal recoveries.
 #  This model allows recoveries outside the stratum of release and error in the smoothed U curve.
@@ -31,35 +55,40 @@ TimeStratPetersenNonDiagErrorNP <- function(title, prefix, time, n1, m2, u2,
 #  Of the n1 fish released, some are recapturd in this stratum of release (column 1 of m2) or in
 #     subsequent strata (subsequent columns of m2). No fish are assumed to be available for capture
 #     outside the range of strata considered in the matrix of m2
-#  At the same tine, u2 other (unmarked) fish are newly captured in stratum i. 
+#  At the same tine, u2 other (unmarked) fish are newly captured in stratum i.
 #     These EXCLUDE recaptures of marked fish. These are the fish that are "expanded"
-#     to estimate the population size of fish in stratum i.  
-# 
+#     to estimate the population size of fish in stratum i.
+#
 #  Input
 #      prefix - prefix for file name for initial plot of U's
 #      time- the stratum number
 #      n1  - vector of number of fish released in stratum i
 #      m2  - matrix of number of fish recovered who were released in stratum i and recovered in stratum j
 #      u2  - vector of number of unmarked fish captured in stratum i
-#      jump.after - points after which the spline is allowed to jump. Specify as a list of integers in the 
+#      jump.after - points after which the spline is allowed to jump. Specify as a list of integers in the
 #              range of 1:Nstrata. If jump.after[i]=k, then the spline is split between strata k and k+1
 #      logitP.cov - covariates for logit(P)=X beta.logitP.cov
-#                 - specify anything you want for fixed logitP's as the covariate values are simply ignored. 
+#                 - specify anything you want for fixed logitP's as the covariate values are simply ignored.
 #                 - recommend that you specify 1 for the intercept and 0's for everything else
-#      logitP.fixed - values for logitP that are fixed in advance. Use NA if corresponding value is not fixed, 
+#      logitP.fixed - values for logitP that are fixed in advance. Use NA if corresponding value is not fixed,
 #                    otherwise specify the logitP value.
 
 
-#  This routine makes a call to WinBugs to fit the model and then gets back the 
+#  This routine makes a call to WinBugs to fit the model and then gets back the
 #  coda files for the posteriour distribution.
 
-library("R2WinBUGS")  # Make sure that all the packages needed are available
-library("coda")       # used for convergence diagnostics
-library("splines")    # used to compute the Bspline design matrix
+## Set working directory to current directory (we should allow users to select this)
+working.directory <- getwd()
+
+## Define paths for the model, data, and initial value files
+model.file <- file.path(working.directory, "model.txt")
+data.file <- file.path(working.directory,"data.txt")
+init.files <- file.path(working.directory,
+                       paste("inits", 1:n.chains,".txt", sep = ""))
 
 # Save the WinBugs progam to the model.txt file
 #
-sink("model.txt")  # NOTE: NO " allowed in model as this confuses the cat command
+sink(model.file)  # NOTE: NO " allowed in model as this confuses the cat command
 cat("
 
 model TimeStratPetersenNonDiagErrorNP{
@@ -99,8 +128,6 @@ model TimeStratPetersenNonDiagErrorNP{
 #                       - the other beta terms are given a prior of a N(mu=0, variance=1000)
 #      tauP.alpha, tauP.beta - parameter for prior on tauP (residual variance of logit(P)'s after adjusting for
 #                         covariates)
-#      xiMu, tauMu  - mean and precision (1/variance) for prior on mean(log travel-times)
-#      siSd, tauSd  - mean and precision (1/variance) for prior on sd(log travel times)
 #
 #  Parameters of the model are:
 #      p[i]
@@ -111,24 +138,28 @@ model TimeStratPetersenNonDiagErrorNP{
 #      U[i]
 #       etaU[i]  = log(U[i])
 #         which comes from spline with parameters bU[1... Knots+q]
-#         + error term eU[i] 
+#         + error term eU[i]
 #
 #      muTT[j] = mean(logit(delta[i,i+j-1])), j=1,...,Delta.max
 #      sdTT = sd(logit(delta[i,i+j-1])), j=1,....,Delta.max
 #      delta[i,i+j-1]=Theta[i,i+j-1]/(1-Theta[i,i]-...-Theta[i,i+j-2])
-#       
+#
 
    ##### Fit the spline for the U's and specify hierarchial model for the logit(P)'s ######
    for(i in 1:(Nstrata.cap)){
         ## Model for U's
-        logUne[i] <- inprod(SplineDesign[i,1:n.bU],bU[1:n.bU])  # spline design matrix * spline coeff 
-        etaU[i] ~ dnorm(logUne[i], taueU)              # add random error
+        logUne[i] <- inprod(SplineDesign[i,1:n.bU],bU[1:n.bU])  # spline design matrix * spline coeff
+        etaU[i] ~ dnorm(logUne[i], taueU)C(,20)              # add random error but keep from getting too large
         eU[i] <- etaU[i] - logUne[i]
    }
 
    for(i in 1:Nfree.logitP){   # model the free capture rates using covariates
-        mu.logitP[free.logitP.index[i]] <- inprod(logitP.cov[free.logitP.index[i],1:NlogitP.cov], beta.logitP[1:NlogitP.cov]) 
-        logitP[free.logitP.index[i]] ~ dnorm(mu.logitP[free.logitP.index[i]],tauP)
+        mu.logitP[free.logitP.index[i]] <- inprod(logitP.cov[free.logitP.index[i],1:NlogitP.cov], beta.logitP[1:NlogitP.cov])
+        ## logitP[free.logitP.index[i]] ~ dnorm(mu.logitP[free.logitP.index[i]],tauP)
+        mu.epsilon[free.logitP.index[i]] <- mu.logitP[free.logitP.index[i]] - log(u2[free.logitP.index[i]] + 1) + etaU[free.logitP.index[i]]
+        epsilon[free.logitP.index[i]] ~ dnorm(mu.epsilon[free.logitP.index[i]],tauP)
+
+        logitP[free.logitP.index[i]] <- log(u2[free.logitP.index[i]] + 1) - etaU[free.logitP.index[i]] + epsilon[free.logitP.index[i]]
    }
 
    ##### Priors and hyperpriors #####
@@ -139,25 +170,26 @@ model TimeStratPetersenNonDiagErrorNP{
         ## is assumed to have a normal distribution with mean muTT[j] and precision tauTT.
 
         r[i,1] ~ dnorm(muTT[1],tauTT)
-		
+
 	logit(Theta[i,1] ) <- r[i,1]
-		
+
 	for(j in 2:Delta.max){
 		r[i,j] ~ dnorm(muTT[j],tauTT)
-			
+
 		logit(delta[i,j]) <- r[i,j]
-			
+
 		Theta[i,j] <- delta[i,j] * (1 - sum(Theta[i,1:(j-1)]))
 	}
 	Theta[i,Delta.max+1] <- 1- sum(Theta[i,1:Delta.max])
    }
 
+#  prior on the movement rates. These are specified using the make.muTT.prior function
    for(j in 1:Delta.max){
-	muTT[j] ~ dnorm(0,.666)
+	muTT[j] ~ dnorm(mean.muTT[j],tau.muTT[j])
    }
    tauTT~ dgamma(tauTT.alpha,tauTT.beta)
    sdTT <- 1/sqrt(tauTT)
-	
+
    ## Run size - flat priors
    for(i in 1:n.b.flat){
       bU[b.flat[i]] ~ dflat()
@@ -230,7 +262,7 @@ SplineDesign <- matrix(0, nrow=0, ncol=0)
 SplineDegree <- 3           # Degree of spline between occasions
 b.flat <- NULL              # index of spline coefficients with a flat prior distribution -first two of each segment
 b.notflat <- NULL           # index of spline coefficients where difference is modelled
-all.knots <- NULL    
+all.knots <- NULL
 
 for (i in 1:(length(ext.jump)-1)){
   nstrata.in.set <- ext.jump[i+1]-ext.jump[i]
@@ -241,7 +273,7 @@ for (i in 1:(length(ext.jump)-1)){
     }
   all.knots <- c(all.knots, knots)
   # compute the design matrix for this set of strata
-  z <- bs((1:nstrata.in.set)/(nstrata.in.set+1), knots=knots, degree=SplineDegree, 
+  z <- bs((1:nstrata.in.set)/(nstrata.in.set+1), knots=knots, degree=SplineDegree,
              intercept=TRUE, Boundary.knots=c(0,1))
   # first two elements of b coeffients have a flat prior
   b.flat <- c(b.flat, ncol(SplineDesign)+(1:2))
@@ -257,7 +289,7 @@ n.b.notflat <- length(b.notflat)
 n.bU <- n.b.flat + n.b.notflat
 
 
-# get the logitP covariate matrix ready 
+# get the logitP covariate matrix ready
 logitP.cov <- as.matrix(logitP.cov)
 NlogitP.cov <- ncol(as.matrix(logitP.cov))
 
@@ -267,12 +299,14 @@ storage.mode(logitP) <- "double"  # force the storage class to be correct if the
 free.logitP.index <- (1:Nstrata.cap)[ is.na(logitP.fixed)]  # free values are those where NA is specifed
 Nfree.logitP <- length(free.logitP.index)
 
+tau.muTT <- 1/sd.muTT**2  # convert from sd to precision = 1/variance
 datalist <- list("Nstrata.rel", "Nstrata.cap","Extra.strata.cap",
                  "Delta.max","n1", "m2", "u2",
                  "logitP", "Nfree.logitP", "free.logitP.index",
                  "logitP.cov", "NlogitP.cov",
                  "SplineDesign",
                  "b.flat", "n.b.flat", "b.notflat", "n.b.notflat", "n.bU",
+                 "mean.muTT", "tau.muTT", # priors on muTT
                  "tauTT.alpha","tauTT.beta",
                  "tauU.alpha", "tauU.beta", "taueU.alpha", "taueU.beta",
                  "mu_xiP", "tau_xiP", "tauP.alpha", "tauP.beta")
@@ -292,7 +326,9 @@ if(debug2) {
 }
 
 ## 2) Capture probabilities
-logitPguess <- c(logit((apply(m2[,1:(Delta.max+1)],1,sum)+1)/(n1+1)),rep(mu_xiP,Nstrata.cap-Nstrata.rel))
+logitPguess <- c(logit(pmin(.99,pmax(.01,(apply(m2[,1:(Delta.max+1)],1,sum)+1)/(n1+1)))),
+                 rep(mu_xiP,Nstrata.cap-Nstrata.rel))
+#browser()
 init.beta.logitP <- as.vector(lm( logitPguess ~ logitP.cov-1)$coefficients)
 if(debug2) {
    cat(" obtained initial values of beta.logitP\n")
@@ -302,34 +338,34 @@ if(debug2) {
 
 # create an initial plot of the fit
 pdf(file=paste(prefix,"-initialU.pdf",sep=""))
-plot(time, log(Uguess[1:Nstrata.cap]), 
+plot(time, log(Uguess[1:Nstrata.cap]),
     main=paste(title,"\nInitial spline fit to estimated U[i]"),
     ylab="log(U[i])", xlab='Stratum')  # initial points on log scale.
 lines(time, SplineDesign %*% init.bU)  # add smoothed spline through points
 dev.off()
 
 parameters <- c("logitP", "beta.logitP", "tauP", "sigmaP",
-                "bU", "tauU", "sigmaU", 
-                "eU", "taueU", "sigmaeU", 
+                "bU", "tauU", "sigmaU",
+                "eU", "taueU", "sigmaeU",
                 "Ntot", "Utot", "logUne", "etaU", "U",
                  "muTT","sdTT","Theta")
 
 if( any(is.na(m2))) {parameters <- c(parameters,"m2")} # monitor in case some bad data where missing values present
 if( any(is.na(u2))) {parameters <- c(parameters,"u2")}
-                 
+
 init.vals <- function(){
    init.logitP <- c(logit((apply(m2[,1:(Delta.max+1)],1,sum)+1)/(n1+1)),rep(mu_xiP,Nstrata.cap-Nstrata.rel))         # initial capture rates based on observed recaptures
    init.logitP <- pmin(10,pmax(-10,init.logitP))
    init.logitP[is.na(init.logitP)] <- -2         # those cases where initial probability is unknown
    init.logitP[!is.na(logitP.fixed)] <- NA        # no need to initialize the fixed values
-   
+
    init.beta.logitP <- as.vector(lm( init.logitP ~ logitP.cov-1)$coefficients)
-   init.beta.logitP[is.na(init.beta.logitP)] <- 0 
-   init.beta.logitP <- c(init.beta.logitP, 0)   # add one extra element so that single beta is still written as a 
+   init.beta.logitP[is.na(init.beta.logitP)] <- 0
+   init.beta.logitP <- c(init.beta.logitP, 0)   # add one extra element so that single beta is still written as a
                                              # vector in the init files etc.
 
    init.logitP <- c(init.logitP,rep(NA,Extra.strata.cap)) # Add values for extra capture probabilities
-   
+
    init.tauP <- 1/var(init.logitP, na.rm=TRUE)     # 1/variance of logit(p)'s (ignoring the covariates for now)
 
    init.bU   <- lm(log(Uguess) ~ SplineDesign-1)$coefficients  # initial values for spline coefficients
@@ -356,147 +392,58 @@ init.vals <- function(){
         return(rep(NA,Delta.max+1))
 
      else{
-       thetatmp <- pmax(.01,pmin(m2[i,-Delta.max+2]/sum(m2[i,-Delta.max+2]),.99))
+       thetatmp <- pmax(.01,pmin(m2[i,-(Delta.max+2)]/sum(m2[i,-(Delta.max+2)],na.rm=TRUE),.99,na.rm=TRUE))  # CJS 2011-02-16
        return(thetatmp/sum(thetatmp))
      }
    }))
-
-   init.delta <- t(apply(init.Theta[,-Delta.max+1],1,function(theta){
-     theta/(1-c(0,cumsum(theta[-Delta.max])))
+# cat('Initial values')
+# browser()
+   init.delta <- t(apply(as.matrix(init.Theta[,-(Delta.max+1)]),1,   # CJS 2011-02-16 as.matrix added
+      function(theta){    # CJS fixed -(Delta.max+1)
+         if(length(theta) == 1){theta} else {theta/(1-c(0,cumsum(theta[-Delta.max])))}
    }))
-   
+
    ## mean and standard deviation of transition probabilties
    init.muTT <- apply(logit(init.delta),2,mean,na.rm=TRUE)
    init.sdTT <- sd(as.vector(t(logit(init.delta)))-init.muTT,na.rm=TRUE)
-   
-   list(logitP=init.logitP, beta.logitP=init.beta.logitP, tauP=init.tauP, 
-        bU=init.bU,  tauU=init.tauU, taueU=init.taueU, etaU=init.etaU, 
+
+#browser()
+   list(logitP=init.logitP, beta.logitP=init.beta.logitP, tauP=init.tauP,
+        bU=init.bU,  tauU=init.tauU, taueU=init.taueU, etaU=init.etaU,
         muTT=init.muTT, tauTT=1/init.sdTT^2,r=logit(init.delta))
 }
 
+init.vals <- genInitVals("TSPNDENP",
+                         n1,m2,u2,
+                         Delta.max=Delta.max,
+                         logitP.cov=logitP.cov,
+                         logitP.fixed=logitP.fixed,
+                         SplineDesign=SplineDesign,
+                         n.chains=n.chains)
 #browser()
+## Generate data list
+data.list <- list()
+for(i in 1:length(datalist)){
+  data.list[[length(data.list)+1]] <-get(datalist[[i]])
+}
+names(data.list) <- as.list(datalist)
 
+# Set up for the call to WinBUGS or OpenBUGS
 
-# set up for the call to WinBugs or OpenBugs
+results <- run.openbugs(modelFile=model.file,
+                        dataFile=data.file,
+                        dataList=data.list,
+                        initFiles=init.files,
+                        initVals=init.vals,
+                        parameters=parameters,
+                        nChains=n.chains,
+                        nIter=n.iter,
+                        nBurnin=n.burnin,
+                        nSims=n.sims,
+                        overRelax=FALSE,
+                        initialSeed=InitialSeed,
+                        working.directory=working.directory,
+                        debug=debug)
 
-if(openbugs){
-   cat("\n\n*** Start of call to OpenBugs \n")
-   bugs.directory = OPENBUGS.directory 
-   # the following call sequence is mainly based on the "openbugs" function in the R2WinBugs package
-   modelFile <- "model.txt"  # this was written in the current directory earlier
-   numChains <- n.chains
-   nBurnin   <- n.burnin
-   nIterPostBurnin <- n.iter - n.burnin
-   nThin     <- round(nIterPostBurnin/n.sims)  # want certain # of final samples so decide upon the thining rate
-   over.relax <- FALSE
-   parameters.to.save <- c(parameters, "deviance")  # always get the deviance
-   parametersToSave <- parameters.to.save
-   working.directory = getwd()          # store all data and init files in the current directory
-
-   cat("OpenBugs files created in:", working.directory, "\n")
-   BRugs::modelCheck(modelFile)    # check the model
-
-   ## If extra columns were added to the data then we need to add fixed capture probabilities at 0.
-   ##    if(m2.excols>0){
-   ##      logitP <- c(logitP,rep(-10,m2.excols))
-   ##    }
-   
-   ## get the data and write to the data.txt file in the current directory
-   datafileName = file.path(working.directory, "data.txt")
-   data.list <- list()
-   for(i in 1:length(datalist)){
-     data.list[[length(data.list)+1]] <- get(datalist[[i]])
-   }
-
-   names(data.list) <- as.list(datalist)
-   
-   
-   temp <- BRugs::bugsData(data.list, fileName = datafileName, digits = 5)
-   cat("Data files saved in ", temp, "\n")
-
-   BRugs::modelData(datafileName)
-   cat("Data loaded into model\n")
-   BRugs::modelCompile(numChains)
-
-   # set the random number generator seed
-   BRugs::modelSetSeed(InitialSeed)
-   cat("Random seed initialized with :", InitialSeed, "\n")
-
-   # generate the files to save the initial values
-   initfileNames <- file.path(working.directory, paste("inits", 1:numChains,".txt", sep = ""))
-   inits <- BRugs::bugsInits(inits = init.vals, numChains = numChains, fileName=initfileNames)
-   cat("Initial values generated in ",inits, "\n")
-   BRugs::modelInits(inits)
-   BRugs::modelGenInits()     # generate the initial values for any uninitialized variables
-   cat("Initial values loaded into model\n")
-
-   # now to generate the burnin sample
-   cat("Burnin sampling has been started for ", nBurnin, " iterations.... \n")
-   flush.console()
-   for(iter in seq(1,nBurnin,round(nBurnin/20)+1)){  # generate a report about every 5% of the way
-      cat('... Starting burnin iteration', iter,' which is about ',round(iter/nBurnin*100),"% of the burnin phase at ",date(),"\n")
-      flush.console()
-      BRugs::modelUpdate(round(nBurnin/20)+1, overRelax = over.relax)
-   }
-   cat("Burnin sampling completed \n")
-
-   # generate the non-burnin samples
-   BRugs::dicSet()      # turn on DIC computations
-   on.exit(BRugs::dicClear(), add = TRUE) 
-   cat("DIC collection set \n")  
-   BRugs::samplesSet(parametersToSave)
-   cat("Nodes to monitor set\n")
-   cat("Starting sampling after burnin for ", n.chains," chain each with  a further ", 
-        nIterPostBurnin, " iterations. \n A thining rate of ", nThin, 
-        "will give about ", round(nIterPostBurnin/nThin), " posterior values in each chain... \n")
-   for(iter in seq(1,nIterPostBurnin,round(nIterPostBurnin/20)+1)){
-      cat('... Starting post-burnin iteration', iter,' which is about ',round(iter/nIterPostBurnin*100),"% of the post-burnin phase at ",date(),"\n")
-      flush.console()
-      BRugs::modelUpdate(round(nIterPostBurnin/nThin/20)+1, thin=nThin, overRelax = over.relax) # we do the thining on the fly
-   }
-   cat("Finished sampling after burnin and thining \n")
-   FinalSeed <- BRugs::modelGetSeed(i=1)
-   cat("Random seed ended with :", FinalSeed, "\n")
- 
-   # Now to extract the sampled values and create the bugs array
-   cat("Extracting the sampled values\n")
-   params <- BRugs::samplesMonitors("*")
-   samples <- sapply(params, BRugs::samplesSample)
-   n.saved.per.chain <- nrow(samples)/numChains
-   samples.array <- array(samples, c(n.saved.per.chain, numChains, ncol(samples)))    
-   dimnames(samples.array)[[3]] <- dimnames(samples)[[2]]
-   DICOutput <- BRugs::dicStats()
-
-   # save the information. The simulation has already been thinned, so no need to thin again
-   results<- as.bugs.array(sims.array = samples.array, 
-        model.file = modelFile, program = "OpenBUGS", DIC = TRUE, 
-        DICOutput = DICOutput, n.iter = n.iter, n.burnin = n.burnin, 
-        n.thin = nThin)
-   results$Seed.initial <- InitialSeed
-   results$Seed.final   <- FinalSeed
-   cat("Final dimension of saved simulation output is ", dim(results$sims.array), "\n")
-
-   # save the information to the coda files
-   BRugs::samplesCoda("*", stem=paste(working.directory,"/",sep=""))  # write out code files
-   cat("Coda file created \n")
-
-   cat("\n\n*** Finished OpenBugs ***\n\n")
-   results
-   } else {
-   bugs.directory = WINBUGS.directory  
-   results <- bugs( 
-      datalist,   # notice this is a list of external variables 
-      inits=init.vals, # function to generate initial values for each chain
-      parameters,
-      model.file="model.txt",
-      n.chains=n.chains,
-      n.iter=n.iter,    # includes burn in
-      n.burnin=n.burnin,
-      n.sims=n.sims,      # (approx) number of final simulations to save
-      bugs.directory=bugs.directory,
-      debug=debug     )
-   results
-   } # end of else clause
-
-
-} # end of TimeStratPetersenNonDiagError function
+return(results)
+}
