@@ -36,9 +36,9 @@
 # In this implementation, the parameters of the log-normal movement distribution are
 # smoothed over time.
 
-
 demo.proceed <- TRUE
-demo.ans <- " "
+if(!exists("demo.ans")) demo.ans <- " "
+
 while(! demo.ans %in% c("yes","no","YES","NO","Y","N","y","n")){
   cat("***** WARNING ***** This demonstration may create/over-write objects with names 'demo.xxx' \n")
   cat("***** WARNING ***** This demonstration may create/over-write a directory 'demo-TSPNDE' \n")
@@ -91,7 +91,6 @@ demo.data <- textConnection(
 
 demo.Fish <- read.csv(demo.data, header=TRUE)
 
-
 demo.Fish[1:5,]
 
 # Now to extract the subset of data, do any fancy adjustments, and fit the data.
@@ -152,9 +151,7 @@ demo.cr.2009.as.tspnde <- TimeStratPetersenNonDiagError_fit(
                   bad.u2=     demo.bad.u2,
                   logitP.fixed=demo.logitP.fixed,
                   logitP.fixed.values=demo.logitP.fixed.values,
-                  debug=TRUE, 
-                  openbugs=TRUE
-                  )
+                  debug=TRUE)
 
 # Rename files that were created.
 # Note that if WinBugs is used, the files are called coda1, coda2, coda3 
